﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Notebook : MonoBehaviour
{
    //Get a reference to the EvidenceHandler in the level.
    EvidenceHandler _handler;

    //List of all buttons and associated evidence.
    public List<Button> _corOptions;
    //List<Evidence> _foundEvidence;

    //List of selected corroborations.
    public List<Evidence> _selected;

    //Used to setup all the prefabs and parents in the inspector
    public GameObject clue;
    public GameObject cluePanel;
    public GameObject clueButton;
    public GameObject clueCorPanel;
    public GameObject statement;
    public GameObject statementPanel;
    public GameObject statementButton;
    public GameObject statementCorPanel;
    public GameObject cor;
    public GameObject corPanel;
    public GameObject corButton;
    public GameObject corCorPanel;

    //bool that says whether corroborations have been created before. Used for the evidence creation tutorial.
    public bool corTutCheck;

    // Start is called before the first frame update
    void Start()
    {
        _handler = FindObjectOfType<EvidenceHandler>();

        //initialise lists
        _corOptions = new List<Button>();
        //_foundEvidence = new List<Evidence>();
        _selected = new List<Evidence>();

        corTutCheck = false;
    }

    // Update is called once per frame
    void Update()
    {
        //if 2 pieces of evidence have been selected:
        if (_selected.Count == 2)
        {
            // tests the 2 pieces of evidence in the handler
            Evidence corroboration = _handler.testCorroboration(_selected[0], _selected[1]);

            // if the returned corroboration isnt null, that means a new corroboration was found.
            if (corroboration.type != Evidence.evidenceType.NullEvidence)
            {
                // adds the new corrroboration to the found list.
                _handler.foundCorroborations.Add(corroboration);

                // adds the new corroboration to the notebook ui.
                AddCorroborations(corroboration);
            }

            // clears the selected list no matter what
            _selected.Clear();
        }
    }

    void addSelected(string id)
    {
        _selected.Add(_handler.GetEvidenceFromID(id));
    }

    public void AddClues(Evidence e)
    {
        // instantiates new clue element in the notebook ui
        GameObject newClue = Instantiate(clue) as GameObject;
        Text[] clueInfo = newClue.GetComponentsInChildren<Text>();
        clueInfo[0].text = e.id;
        clueInfo[1].text = e.details;
        newClue.transform.SetParent(cluePanel.transform, false);
        newClue.GetComponent<EvidenceUI>().thisEvidenceID = e.id;

        // instantiates the same element but with a button in the corroboration screen
        GameObject newClueButton = Instantiate(clueButton) as GameObject;
        Text[] clueButtonInfo = newClueButton.GetComponentsInChildren<Text>();
        clueButtonInfo[0].text = e.id;
        clueButtonInfo[1].text = e.details;
        newClueButton.transform.SetParent(clueCorPanel.transform, false);
        newClueButton.GetComponent<EvidenceUI>().thisEvidenceID = e.id;

        // adds the new button and evidence to lists.
        newClueButton.GetComponent<Button>().onClick.AddListener(() => addSelected(e.id));
        _corOptions.Add(newClueButton.GetComponent<Button>());
    }

    public void AddStatements(Evidence e)
    {
        // instantiates new statement element in the notebook ui
        GameObject newStatement = Instantiate(statement) as GameObject;
        Text[] statementInfo = newStatement.GetComponentsInChildren<Text>();
        statementInfo[0].text = e.id;
        statementInfo[1].text = e.details;
        newStatement.transform.SetParent(statementPanel.transform, false);
        newStatement.GetComponent<EvidenceUI>().thisEvidenceID = e.id;

        // instantiates the same element but with a button in the corroboration screen
        GameObject newStatementButton = Instantiate(statementButton) as GameObject;
        Text[] statementButtonInfo = newStatementButton.GetComponentsInChildren<Text>();
        statementButtonInfo[0].text = e.id;
        statementButtonInfo[1].text = e.details;
        newStatementButton.transform.SetParent(statementCorPanel.transform, false);
        newStatementButton.GetComponent<EvidenceUI>().thisEvidenceID = e.id;

        // adds the new button and evidence to lists.
        newStatementButton.GetComponent<Button>().onClick.AddListener(() => addSelected(e.id));
        _corOptions.Add(newStatementButton.GetComponent<Button>());
    }

    public void AddCorroborations(Evidence e)
    {
        // instantiates new corroborration element in the notebook ui
        GameObject newCor = Instantiate(cor) as GameObject;
        Text[] corInfo = newCor.GetComponentsInChildren<Text>();
        corInfo[0].text = e.id;
        corInfo[1].text = e.details;
        newCor.transform.SetParent(corPanel.transform, false);
        newCor.GetComponent<EvidenceUI>().thisEvidenceID = e.id;


        // instantiates the same element but with a button in the corroboration screen
        GameObject newCorButton = Instantiate(corButton) as GameObject;
        Text[] corButtonInfo = newCorButton.GetComponentsInChildren<Text>();
        corButtonInfo[0].text = e.id;
        corButtonInfo[1].text = e.details;
        newCorButton.transform.SetParent(corCorPanel.transform, false);
        newCorButton.GetComponent<EvidenceUI>().thisEvidenceID = e.id;


        // adds the new button and evidence to lists.
        newCorButton.GetComponent<Button>().onClick.AddListener(() => addSelected(e.id));
        _corOptions.Add(newCorButton.GetComponent<Button>());
        //_foundEvidence.Add(e);

        corTutCheck = true;
    }
}
